//===========================================================
// Dde32.dll - Ddeml wrapper library
//
// Copyright (C) 1997 Pivato Consulting, inc.
// Author: Gianluca Pivato
//
// Revision History
//
//  1.0		Initial Release
//	2.0		Converted to 32bit
//===========================================================

//-----------------------------------------------------------
// Include files
//-----------------------------------------------------------
#include "windows.h"
#include "ddeml.h"


//============================================================
// Declarations
//============================================================
UINT WINAPI DdeWndInitialize( DWORD FAR* lpidInst, DWORD afCmd, DWORD uRes );
HDDEDATA CALLBACK DDECallback( UINT type, UINT fmt, HCONV hconv, 
								HSZ hsz1, HSZ hsz2, HDDEDATA hData, DWORD dwData1, DWORD dwData2 );
HCONV WINAPI DdeWndConnect( DWORD idInst, HSZ hszService, HSZ hszTopic, CONVCONTEXT FAR* pCC, HWND hWnd );
BOOL WINAPI DdeWndDisconnect( HCONV hConv );

int FindConv( HCONV hConv );

//============================================================
// DLL Globals and definitions
//============================================================

#define MAXCONV 64
/*	Sent when the dde-server closes the conversation.
			wParam	= not used
			lParam	= conversation handle
*/
#define DDE_DISCONNECTED	(WM_USER + 10)
/*	Sent when the value of an item in an advise loop is changed.
				wParam	= TRUE the message has data
				lParam	= data handle or null
*/
#define DDE_DATACHANGE		(WM_USER + 11)
/*	Sent when an asyncronous transaction is completed.
				wParam	= not used
				lParam	= conversation handle
*/
#define DDE_ASYNCOMPLETED	(WM_USER + 12)


static HWND g_hWndPool[MAXCONV];
static HCONV g_hConvPool[MAXCONV];
static HINSTANCE g_hInst=NULL;

//============================================================
// LibMain -- Entry point called during DLL init
//============================================================
BOOL APIENTRY DllMain ( HINSTANCE hInstance, 
                       ULONG ul_reason_for_call,
                       LPVOID lpReserved ) {

	int idx;
	
	// Initialize the list
	if ( !g_hInst ) {
		g_hInst = hInstance;
		
		for ( idx=0; idx<MAXCONV; idx++ ) {
			g_hConvPool[idx] = NULL;
			g_hWndPool[idx] = NULL;
		}
	}
		
    
	return TRUE;
}


//============================================================
// DdeWndInitialize -- Wraps DdeInitialize
//============================================================
UINT WINAPI DdeWndInitialize( DWORD FAR* lpidInst, DWORD afCmd, DWORD uRes ) {

	// Initialize the library
	return DdeInitialize( lpidInst, 
							(PFNCALLBACK) GetProcAddress( g_hInst, "DDECallback" ), 
								afCmd, 
									uRes );

}

//============================================================
// DdeWndConnect -- Wraps DeeConnect
//============================================================
HCONV WINAPI DdeWndConnect( DWORD idInst, HSZ hszService, HSZ hszTopic, CONVCONTEXT FAR* pCC, HWND hWnd ) {

	HCONV hConv;
	int idx;
	
	// Find a free item in the list, return an error if there isn't any
	idx = FindConv( 0 );
	if ( idx < 0 ) return NULL;
	
	// Connect
	hConv = DdeConnect( idInst, hszService, hszTopic, pCC );
	if ( !hConv ) return hConv;
	
	// Bind the window that has to receive the notification messages
	if ( IsWindow( hWnd ) ) {
		g_hConvPool[idx] = hConv;
		g_hWndPool[idx] = hWnd;
	}

	return hConv;

}

//============================================================
// DdeWndDisconnect -- Wraps DeeDisconnect
//============================================================
BOOL WINAPI DdeWndDisconnect( HCONV hConv ) {

	int idx;
	
	// Clear the conversation item in the list
	idx = FindConv( hConv );
	if ( idx > -1 ) {
		g_hConvPool[idx] = NULL;
		g_hWndPool[idx] = NULL;
	}
	
	return DdeDisconnect( hConv );

}

//============================================================
// DDECallback -- Deeml callback function
//============================================================
HDDEDATA CALLBACK DDECallback( UINT type, UINT fmt, HCONV hconv, 
								HSZ hsz1, HSZ hsz2, HDDEDATA hData, 
								DWORD dwData1, DWORD dwData2 ) {


	int idx, ret;
	
	// Find the conversation in the list
	idx = FindConv( hconv );
	if ( idx < 0 ) return (HDDEDATA) NULL;
	
		
	// Send the message to the corresponding window
	switch ( type ) {
	
		case XTYP_DISCONNECT:
			PostMessage( g_hWndPool[idx], DDE_DISCONNECTED, 0, (LPARAM) hconv );
			break;
			
		case XTYP_XACT_COMPLETE:
			PostMessage( g_hWndPool[idx], DDE_ASYNCOMPLETED, 0, (LPARAM) hconv );
			break;

		case XTYP_ADVDATA:
			ret = (int) SendMessage( g_hWndPool[idx], DDE_DATACHANGE, 
									(WPARAM) (hData != NULL), (LPARAM) hData );
			if ( ret = DDE_FBUSY ) return (HDDEDATA) DDE_FBUSY;
			return (HDDEDATA) DDE_FACK;
			break;
	}
	
	return (HDDEDATA) NULL;
}

//============================================================
// FindConv -- Find a conversation in the list
//============================================================
int FindConv( HCONV hConv ) {

	int idx;

	for ( idx=0; idx<MAXCONV; idx++ )
		if ( g_hConvPool[idx] == hConv ) return idx;
	
	return -1;
}

